@@ -26,10 +26,12 @@ import shutil
 import black
 import logging
 
+from collections import defaultdict
+from collections.abc import MutableSet
 from pathlib import Path
 from functools import partial
 from itertools import zip_longest
-from typing import List, Iterable
+from typing import Dict, Iterable, List, Optional
 from black import InvalidInput
 
 try:
@@ -39,6 +41,7 @@ except ImportError:
 
 from yaml import safe_dump as dump
 
+import sphinx.application
 from sphinx.util.console import darkgreen, bold
 from sphinx.util import ensuredir
 from sphinx.errors import ExtensionError
@@ -51,6 +54,7 @@ from .settings import API_ROOT
 from .monkeypatch import patch_docfields
 from .directives import RemarksDirective, TodoDirective
 from .nodes import remarks
+from docfx_yaml import markdown_utils
 
 import subprocess
 import ast
@@ -83,12 +87,12 @@ REFMETHOD = 'meth'
 REFFUNCTION = 'func'
 INITPY = '__init__.py'
 # Regex expression for checking references of pattern like ":class:`~package_v1.module`"
-REF_PATTERN = ':(py:)?(func|class|meth|mod|ref|attr|exc):`~?[a-zA-Z0-9_\.<> ]*(\(\))?`'
+REF_PATTERN = ':(py:)?(func|class|meth|mod|ref|attr|exc):`~?[a-zA-Z0-9_.<> ]*(\(\))?`'
 # Regex expression for checking references of pattern like "~package_v1.subpackage.module"
 REF_PATTERN_LAST = '~([a-zA-Z0-9_<>]*\.)*[a-zA-Z0-9_<>]*(\(\))?'
 # Regex expression for checking references of pattern like
 # "[module][google.cloud.cloudkms_v1.module]"
-REF_PATTERN_BRACKETS = '\[[a-zA-Z0-9\_\<\>\-\. ]+\]\[[a-zA-Z0-9\_\<\>\-\. ]+\]'
+REF_PATTERN_BRACKETS = '\[[a-zA-Z0-9_<>\-. ]+\]\[[a-zA-Z0-9_<>\-. ]+\]'
 
 REF_PATTERNS = [
     REF_PATTERN,
@@ -101,32 +105,40 @@ CODEBLOCK = "code-block"
 CODE = "code"
 PACKAGE = "package"
 
+# DevSite specific notices that can be used.
+NOTE = 'note'
+CAUTION = 'caution'
+WARNING = 'warning'
+IMPORTANT = 'special'
+KEYPOINT = 'key-point'
+KEYTERM = 'key-term'
+OBJECTIVE = 'objective'
+SUCCESS = 'success'
+BETA = 'beta'
+PREVIEW = 'preview'
+DEPRECATED = 'deprecated'
+
+NOTICES = {
+    NOTE: 'Note',
+    CAUTION: 'Caution',
+    WARNING: 'Warning',
+    IMPORTANT: 'Important',
+    KEYPOINT: 'Key Point',
+    KEYTERM: 'Key Term',
+    OBJECTIVE: 'Objective',
+    SUCCESS: 'Success',
+    BETA: 'Beta',
+    PREVIEW: 'Preview',
+    DEPRECATED: 'deprecated',
+}
+
 # Disable blib2to3 output that clutters debugging log.
 logging.getLogger("blib2to3").setLevel(logging.ERROR)
 
-# Run sphinx-build with Markdown builder in the plugin.
-def run_sphinx_markdown():
-    cwd = os.getcwd()
-    # Skip running sphinx-build for Markdown for some unit tests.
-    # Not required other than to output DocFX YAML.
-    if "docs" in cwd:
-        return
-
-    return shell.run(
-        [
-            "sphinx-build",
-            "-M",
-            "markdown",
-            "docs/",
-            "docs/_build",
-        ],
-        hide_output=False
-    )
-
 
 def build_init(app):
     print("Running sphinx-build with Markdown first...")
-    run_sphinx_markdown()
+    markdown_utils.run_sphinx_markdown()
     print("Completed running sphinx-build with Markdown files.")
 
     """
@@ -151,8 +163,11 @@ def build_init(app):
     app.env.docfx_uid_names = {}
     # This stores file path for class when inspect cannot retrieve file path
     app.env.docfx_class_paths = {}
-    # This stores the name and href of the markdown pages.
-    app.env.markdown_pages = []
+    # This stores the name and href of the nested markdown pages.
+    app.env.markdown_pages = defaultdict(list)
+    # This stores all the markdown pages moved from the plugin, will be used
+    # to compare and delete unused pages.
+    app.env.moved_markdown_pages = set()
 
     app.env.docfx_xrefs = {}
 
@@ -407,6 +422,9 @@ def _parse_docstring_summary(summary):
     attribute_type_token = ":type:"
     keyword = name = description = var_type = ""
 
+    notice_open_tag = '<aside class="{notice_tag}">\n<b>{notice_name}:</b>'
+    notice_close_tag = '</aside>'
+
     # We need to separate in chunks, which is defined by 3 newline breaks.
     # Otherwise when parsing for code and blocks of stuff, we will not be able
     # to have the entire context when just splitting by single newlines.
@@ -468,6 +486,24 @@ def _parse_docstring_summary(summary):
 
                 continue
 
+        elif keyword and keyword in NOTICES:
+            # Determine how much code block is indented to format properly.
+            if tab_space == -1:
+                parts = [split_part for split_part in part.split("\n") if split_part]
+                tab_space = len(parts[0]) - len(parts[0].lstrip(" "))
+                if tab_space == 0:
+                    raise ValueError("Content in the block should be indented."\
+                                     f"Please check the docstring: \n{summary}")
+            if not part.startswith(" "*tab_space):
+                if notice_body:
+                    parts = [indent_code_left(part, tab_space) for part in notice_body]
+                    summary_parts.append("\n".join(parts))
+
+                summary_parts.append(notice_close_tag)
+                keyword = ""
+            notice_body.append(part)
+            continue
+
         # Parse keywords if found.
         # lstrip is added to parse code blocks that are not formatted well.
         if part.lstrip('\n').startswith('..'):
@@ -491,6 +527,24 @@ def _parse_docstring_summary(summary):
                 found_name = False
                 name = part.split("::")[1].strip()
 
+            # Extracts the notice content and format it.
+            elif keyword and keyword in NOTICES:
+                summary_parts.append(notice_open_tag.format(
+                    notice_tag=keyword, notice_name=NOTICES[keyword]))
+                tab_space = -1
+                notice_body = []
+                parts = [split_part for split_part in part.split("\n") if split_part][1:]
+                if not parts:
+                    continue
+                tab_space = len(parts[0]) - len(parts[0].lstrip(" "))
+                if tab_space == 0:
+                    raise ValueError("Content in the block should be indented."\
+                                     f"Please check the docstring: \n{summary}")
+                parts = [indent_code_left(part, tab_space) for part in parts]
+                summary_parts.append("\n".join(parts))
+                summary_parts.append(notice_close_tag)
+                keyword = ""
+
             # Reserve for additional parts
             # elif keyword == keyword:
             else:
@@ -508,6 +562,13 @@ def _parse_docstring_summary(summary):
         if summary_parts[-1] != "```\n":
             summary_parts.append("```\n")
 
+    if keyword and keyword in NOTICES:
+        if notice_body:
+            parts = [indent_code_left(part, tab_space) for part in notice_body]
+            summary_parts.append("\n\n".join(parts))
+        if summary_parts[-1] != notice_close_tag:
+            summary_parts.append(notice_close_tag)
+
     # Requires 2 newline chars to properly show on cloud site.
     return "\n".join(summary_parts), attributes
 
@@ -685,12 +746,42 @@ def _extract_docstring_info(summary_info, summary, name):
             words = []
         else:
             words.append(word)
-    
+
         index += 1
 
     return top_summary
 
 
+def reformat_summary(summary: str) -> str:
+    """Applies any style changes to be made specifically for DocFX YAML.
+
+    Makes the following changes:
+      - converts ``text`` to `text`
+
+    Args:
+        summary: The summary to be modified.
+
+    Returns:
+        Converted summary suitable for DocFX YAML.
+    """
+
+    reformatted_lines = []
+
+    single_backtick = '`'
+    double_backtick = '``'
+    triple_backtick = '```'
+    for line in summary.split('\n'):
+        # Check that we're only looking for double backtick (``) and not
+        # comments (```).
+        if triple_backtick not in line and double_backtick in line:
+            reformatted_lines.append(line.replace(double_backtick, single_backtick))
+
+        else:
+            reformatted_lines.append(line)
+
+    return '\n'.join(reformatted_lines)
+
+
 # Returns appropriate product name to display for given full name of entry.
 def extract_product_name(name):
     if 'google.cloud' in name:
@@ -850,11 +941,6 @@ def _create_datam(app, cls, module, name, _type, obj, lines=None):
         'name': short_name,
         'fullName': name,
         'source': {
-            'remote': {
-                'path': path,
-                'branch': app.env.docfx_branch,
-                'repo': app.env.docfx_remote,
-            },
             'id': short_name,
             'path': path,
             'startLine': start_line,
@@ -880,6 +966,7 @@ def _create_datam(app, cls, module, name, _type, obj, lines=None):
 
         # Extract summary info into respective sections.
         if summary:
+            summary = reformat_summary(summary)
             top_summary = _extract_docstring_info(summary_info, summary, name)
             try:
                 datam['summary'], datam['attributes'] = _parse_docstring_summary(top_summary)
@@ -1020,7 +1107,6 @@ def process_docstring(app, _type, name, obj, options, lines):
         else:
             app.env.docfx_yaml_functions[cls].append(datam)
 
-    insert_inheritance(app, _type, obj, datam)
     insert_children_on_module(app, _type, datam)
     insert_children_on_class(app, _type, datam)
     insert_children_on_function(app, _type, datam)
@@ -1048,25 +1134,6 @@ def process_signature(app, _type, name, obj, options, signature, return_annotati
         app.env.docfx_signature_funcs_methods[name] = signature
 
 
-def insert_inheritance(app, _type, obj, datam):
-
-    def collect_inheritance(base, to_add):
-        for new_base in base.__bases__:
-            new_add = {'type': _fullname(new_base)}
-            collect_inheritance(new_base, new_add)
-            if 'inheritance' not in to_add:
-                to_add['inheritance'] = []
-            to_add['inheritance'].append(new_add)
-
-    if hasattr(obj, '__bases__'):
-        if 'inheritance' not in datam:
-            datam['inheritance'] = []
-        for base in obj.__bases__:
-            to_add = {'type': _fullname(base)}
-            collect_inheritance(base, to_add)
-            datam['inheritance'].append(to_add)
-
-
 def insert_children_on_module(app, _type, datam):
     """
     Insert children of a specific module
@@ -1261,272 +1328,278 @@ def pretty_package_name(package_group):
     return " ".join(capitalized_name)
 
 
-# Check is the current lines conform to markdown header format.
-def parse_markdown_header(header_line, prev_line):
-    # Markdown h1 prefix should have only 1 of '#' character followed by exactly one space.
-    h1_header_prefix = "# "
-    if h1_header_prefix in header_line and header_line.count("#") == 1:
-        # Check for proper h1 header formatting, ensure there's more than just
-        # the hashtag character, and exactly only one space after the hashtag.
-        if not header_line[header_line.index(h1_header_prefix)+2].isspace() and \
-            len(header_line) > 2:
-
-            return header_line[header_line.index(h1_header_prefix):].strip("#").strip()
-
-    elif "=" in header_line:
-        # Check if we're inspecting an empty or undefined lines.
-        if not prev_line:
-            return ""
-
-        # Check if the current line only has equal sign divider.
-        if header_line.count("=") == len(header_line.strip()):
-            # Update header to the previous line.
-            return prev_line.strip()
-
-    return ""
-
-
-def extract_header_from_markdown(mdfile: Iterable[str]) -> str:
-    """For a given markdown file, extract its header line.
+def find_uid_to_convert(
+    current_word: str,
+    words: List[str],
+    index: int,
+    known_uids: List[str],
+    current_object_name: str,
+    processed_words: List[str],
+    hard_coded_references: Dict[str, str] = None
+) -> Optional[str]:
+    """Given `current_word`, returns the `uid` to convert to cross reference if found.
 
     Args:
-        mdfile: iterator to the markdown file.
+        current_word: current word being looked at
+        words: list of words used to check and compare content before and after `current_word`
+        index: index position of `current_word` within words
+        known_uids: list of uid references to look for
+        current_object_name: the name of the current Python object being processed
+        processed_words: list of words containing words that's been processed so far
+        hard_coded_references: Optional list containing a list of hard coded reference
 
     Returns:
-        A string for header or empty string if header is not found.
+        None if current word does not contain any reference `uid`, or the `uid`
+          that should be converted.
     """
-    prev_line = ""
+    for uid in known_uids:
+        # Do not convert references to itself or containing partial
+        # references. This could result in `storage.types.ReadSession` being
+        # prematurely converted to
+        # `<xref uid="storage.types">storage.types</xref>ReadSession`
+        # instead of
+        # `<xref uid="storage.types.ReadSession">storage.types.ReadSession</xref>`
+        if uid in current_object_name:
+            continue
 
-    for header_line in mdfile:
+        if uid in current_word:
+            # If the cross reference has been processed already, "<xref" or
+            # "<a" will appear as the previous word.
+            # For hard coded references, we use "<a href" style.
+            if "<xref" not in words[index-1] and "<a" not in words[index-1]:
+                # Check to see if the reference has been converted already.
+                if not (processed_words and ( \
+                    f"<xref uid=\"{uid}" in processed_words[-1] or \
+                    (hard_coded_references and f"<a href=\"{hard_coded_references.get(uid)}" in processed_words[-1]))):
+                  return uid
 
-        # Ignore licenses and other non-headers prior to the header.
-        header = parse_markdown_header(header_line, prev_line)
-        # If we've found the header, return the header.
-        if header != "":
-            return header
-
-        prev_line = header_line
-
-    return ""
+    return None
 
 
-# For a given markdown file, adds syntax highlighting to code blocks.
-def highlight_md_codeblocks(mdfile):
-    fence = '```'
-    fence_with_python = '```python'
-    new_lines = []
+def convert_cross_references(content: str, current_object_name: str, known_uids: List[str]) -> str:
+    """Finds and replaces references that should be a cross reference in given content.
 
-    with open(mdfile) as mdfile_iterator:
-        file_content = mdfile_iterator.read()
-        # If there is an odd number of code block annotations, do not syntax
-        # highlight.
-        if file_content.count(fence) % 2 != 0:
-            print(f'{mdfile_iterator.name} contains wrong format of code blocks. Skipping syntax highlighting.')
-            return
-        # Retrieve code block positions to replace
-        codeblocks = [[m.start(), m.end()] for m in re.finditer(
-                                                      fence,
-                                                      file_content)]
-
-        # This is equivalent to grabbing every odd index item.
-        codeblocks = codeblocks[::2]
-        # Used to store code blocks that come without language indicators.
-        blocks_without_indicators = []
-
-        # Check if the fence comes without a language indicator. If so, include
-        # this to a list to render.
-        for start, end in codeblocks:
-            if file_content[end] == '\n':
-                blocks_without_indicators.append([start, end])
-
-        # Stitch content that does not need to be parsed, and replace with
-        # `fence_with_python` for parsed portions.
-        prev_start = prev_end = 0
-        for start, end in blocks_without_indicators:
-            new_lines.append(file_content[prev_end:start])
-            new_lines.append(fence_with_python)
-            prev_start, prev_end = start, end
-
-        # Include rest of the content.
-        new_lines.append(file_content[prev_end:])
-
-    # Overwrite with newly parsed content.
-    with open(mdfile, 'w') as mdfile_iterator:
-        new_content = ''.join(new_lines)
-        mdfile_iterator.write(new_content)
-
-
-def prepend_markdown_header(filename: str, mdfile: Iterable[str]):
-    """Prepends the filename as a Markdown header.
+    This should not convert any references that contain `current_object_name`,
+    i.e. if we're processing docstring for `google.cloud.spanner.v1.services`,
+    references to `google.cloud.spanner.v1.services` should not be convereted
+    to references.
 
     Args:
-        filename: the name of the markdown file to prepend.
-        mdfile: iterator to the markdown file that is both readable
-          and writable.
+        content: body of content to parse and look for references in
+        current_object_name: the name of the current Python object being processed
+        known_uids: list of uid references to look for
+
+    Returns:
+        content that has been modified with proper cross references if found.
     """
-    file_content = f'# {filename}\n\n' + mdfile.read()
-    # Reset file position to the beginning to write
-    mdfile.seek(0)
-    mdfile.write(file_content)
-
-
-# Given generated markdown files, incorporate them into the docfx_yaml output.
-# The markdown file metadata will be added to top level of the TOC.
-def find_markdown_pages(app, outdir):
-    # Use this to ignore markdown files that are unnecessary.
-    files_to_ignore = [
-        "index.md",     # merge index.md and README.md and index.yaml later.
-                        # See https://github.com/googleapis/sphinx-docfx-yaml/issues/105.
-
-        "reference.md", # Reference docs overlap with Overview. Will try and incorporate this in later.
-                        # See https://github.com/googleapis/sphinx-docfx-yaml/issues/106.
-
-        "readme.md",    # README does not seem to work in cloud site
-                        # See https://github.com/googleapis/sphinx-docfx-yaml/issues/107.
-    ]
-
-    markdown_dir = Path(app.builder.outdir).parent / "markdown"
-    if not markdown_dir.exists():
-        print("There's no markdown file to move.")
-        return
-
-    # For each file, if it is a markdown file move to the top level pages.
-    for mdfile in markdown_dir.iterdir():
-        if mdfile.is_file() and mdfile.name.lower() not in files_to_ignore:
-            mdfile_name = ""
-            highlight_md_codeblocks(markdown_dir / mdfile.name)
-
-            # Extract the header name for TOC.
-            with open(mdfile) as mdfile_iterator:
-                name = extract_header_from_markdown(mdfile_iterator)
-
-            if not name:
-                with open(mdfile, 'r+') as mdfile_iterator:
-                    mdfile_name = mdfile_iterator.name.split("/")[-1].split(".")[0].capitalize()
-
-                    print(f"Could not find a title for {mdfile_iterator.name}. Using {mdfile_name} as the title instead.")
-                    name = mdfile_name
-
-                    prepend_markdown_header(name, mdfile_iterator)
-
-            shutil.copy(mdfile, f"{outdir}/{mdfile.name.lower()}")
-
-            # Add the file to the TOC later.
-            app.env.markdown_pages.append({
-                'name': name,
-                'href': mdfile.name.lower(),
-            })
-
-
-# Finds and replaces occurrences which should be a cross reference in the given
-# content, except for the current name.
-def convert_cross_references(content: str, current_name: str, entry_names: List[str]):
     words = content.split(" ")
-    new_words = []
-    # Using counter to check if the entry is already a cross reference.
+
+    # Contains a list of words that is not a valid reference or converted
+    # references.
+    processed_words = []
+
+    # TODO(https://github.com/googleapis/sphinx-docfx-yaml/issues/208):
+    # remove this in the future.
+    iam_policy_link = "http://github.com/googleapis/python-grpc-google-iam-v1/blob/8e73b45993f030f521c0169b380d0fbafe66630b/google/iam/v1/iam_policy_pb2_grpc.py"
+    hard_coded_references = {
+        "google.iam.v1.iam_policy_pb2.SetIamPolicyRequest": iam_policy_link + "#L103-L109",
+        "google.iam.v1.iam_policy_pb2.GetIamPolicyRequest": iam_policy_link + "#L111-L118",
+        "google.iam.v1.iam_policy_pb2.TestIamPermissionsRequest": iam_policy_link + "#L120-L131",
+        "google.iam.v1.iam_policy_pb2.TestIamPermissionsResponse": iam_policy_link + "#L120-L131"
+    }
+    known_uids.extend(hard_coded_references.keys())
+
     for index, word in enumerate(words):
-        cross_reference = ""
-        for keyword in entry_names:
-            if keyword != current_name and keyword not in current_name and keyword in word:
-                # If it is already processed as cross reference, skip over it.
-                if "<xref" in words[index-1] or (new_words and f"<xref uid=\"{keyword}" in new_words[-1]):
-                    continue
-                cross_reference = f"<xref uid=\"{keyword}\">{keyword}</xref>"
-                new_words.append(word.replace(keyword, cross_reference))
-                print(f"Converted {keyword} into cross reference in: \n{content}")
+        uid = find_uid_to_convert(
+            word, words, index, known_uids, current_object_name, processed_words, hard_coded_references
+        )
 
-        # If cross reference has not been found, add current unchanged content.
-        if not cross_reference:
-            new_words.append(word)
+        if uid:
+            cross_reference = f"<a href=\"{hard_coded_references[uid]}\">{uid}</a>" \
+                if uid in hard_coded_references else \
+                f"<xref uid=\"{uid}\">{uid}</xref>"
 
-    return " ".join(new_words)
+            processed_words.append(word.replace(uid, cross_reference))
+            print(f"Converted {uid} into cross reference in: \n{content}")
+
+        else:
+            # If cross reference has not been found, add current unchanged content.
+            processed_words.append(word)
+
+    return " ".join(processed_words)
 
 
 # Used to look for cross references in the obj's data where applicable.
 # For now, we inspect summary, syntax and attributes.
-def search_cross_references(obj, current_name: str, entry_names: List[str]):
+def search_cross_references(obj, current_object_name: str, known_uids: List[str]):
     if obj.get("summary"):
-        obj["summary"] = convert_cross_references(obj["summary"], current_name, entry_names)
+        obj["summary"] = convert_cross_references(obj["summary"], current_object_name, known_uids)
 
     if obj.get("syntax"):
         if obj["syntax"].get("parameters"):
             for param in obj["syntax"]["parameters"]:
                 if param.get("description"):
-                    param["description"] = convert_cross_references(
+                    param_description = convert_cross_references(
                         param["description"],
-                        current_name,
-                        entry_names
+                        current_object_name,
+                        known_uids
                     )
+                    param["description"] = (
+                        markdown_utils.reformat_markdown_to_html(param_description))
 
                 if param.get("id"):
-                    param["id"] = convert_cross_references(
+                    param_id = convert_cross_references(
                         param["id"],
-                        current_name,
-                        entry_names
+                        current_object_name,
+                        known_uids
                     )
+                    param["id"] = markdown_utils.reformat_markdown_to_html(param_id)
 
                 if param.get("var_type"):
-                    param["var_type"] = convert_cross_references(
+                    param_type = convert_cross_references(
                         param["var_type"],
-                        current_name,
-                        entry_names
+                        current_object_name,
+                        known_uids
                     )
+                    param["var_type"] = (
+                        markdown_utils.reformat_markdown_to_html(param_type))
 
         if obj["syntax"].get("exceptions"):
             for exception in obj["syntax"]["exceptions"]:
                 if exception.get("description"):
-                    exception["description"] = convert_cross_references(
+                    exception_description = convert_cross_references(
                         exception["description"],
-                        current_name,
-                        entry_names
+                        current_object_name,
+                        known_uids
                     )
+                    exception["description"] = (
+                        markdown_utils.reformat_markdown_to_html(exception_description))
 
                 if exception.get("var_type"):
-                    exception["var_type"] = convert_cross_references(
+                    exception_type = convert_cross_references(
                         exception["var_type"],
-                        current_name,
-                        entry_names
+                        current_object_name,
+                        known_uids
                     )
+                    exception["var_type"] = (
+                        markdown_utils.reformat_markdown_to_html(exception_type))
 
         if obj["syntax"].get("returns"):
             for ret in obj["syntax"]["returns"]:
                 if ret.get("description"):
-                    ret["description"] = convert_cross_references(
+                    ret_description = convert_cross_references(
                         ret["description"],
-                        current_name,
-                        entry_names
+                        current_object_name,
+                        known_uids
                     )
+                    ret["description"] = (
+                        markdown_utils.reformat_markdown_to_html(ret_description))
 
                 if ret.get("var_type"):
-                    ret["var_type"] = convert_cross_references(
+                    ret_type = convert_cross_references(
                         ret["var_type"],
-                        current_name,
-                        entry_names
+                        current_object_name,
+                        known_uids
                     )
+                    ret["var_type"] = markdown_utils.reformat_markdown_to_html(ret_type)
 
 
     if obj.get("attributes"):
         for attribute in obj["attributes"]:
             if attribute.get("description"):
-                attribute["description"] = convert_cross_references(
+                attribute_description = convert_cross_references(
                     attribute["description"],
-                    current_name,
-                    entry_names
+                    current_object_name,
+                    known_uids
                 )
+                attribute["description"] = (
+                    markdown_utils.reformat_markdown_to_html(attribute_description))
 
             if attribute.get("id"):
-                attribute["id"] = convert_cross_references(
+                attribute_id = convert_cross_references(
                     attribute["id"],
-                    current_name,
-                    entry_names
+                    current_object_name,
+                    known_uids
                 )
+                attribute["id"] = markdown_utils.reformat_markdown_to_html(attribute_id)
 
             if attribute.get("var_type"):
-                attribute["var_type"] = convert_cross_references(
+                attribute_type = convert_cross_references(
                     attribute["var_type"],
-                    current_name,
-                    entry_names
+                    current_object_name,
+                    known_uids
                 )
+                attribute["var_type"] = (
+                    markdown_utils.reformat_markdown_to_html(attribute_type))
+
+
+# Type alias used for toc_yaml entries.
+_toc_yaml_type_alias = dict[str, any]
+
+def merge_markdown_and_package_toc(
+    pkg_toc_yaml: list[_toc_yaml_type_alias],
+    markdown_toc_yaml: _toc_yaml_type_alias,
+    known_uids: set[str],
+) -> tuple[MutableSet[str], list[_toc_yaml_type_alias]]:
+    """
+    Merges the markdown and package table of contents.
+
+    Args:
+        pkg_toc_yaml: table of content for package files.
+        markdown_toc_yaml: table fo content for markdown files.
+
+    Returns:
+        A set of markdown pages that has been added, and the merged table of
+        contents file, with files in the correct position.
+    """
+    def _flatten_toc(
+        toc_yaml_entry: list[_toc_yaml_type_alias],
+    ) -> list[_toc_yaml_type_alias]:
+        """Flattens and retrieves all children within pkg_toc_yaml."""
+        entries = list(toc_yaml_entry)
+        for entry in toc_yaml_entry:
+            if (children := entry.get('items')):
+                entries.extend(_flatten_toc(children))
+        return entries
+
+    added_pages = set()
+
+    pkg_toc_entries = _flatten_toc(pkg_toc_yaml)
+
+    for entry in pkg_toc_entries:
+        entry_name = entry['name'].lower()
+        if entry_name not in markdown_toc_yaml:
+            continue
+
+        markdown_pages_to_add = []
+        for page in markdown_toc_yaml[entry_name]:
+            if page['href'].split('.')[0] not in known_uids and (
+                page['href'] not in added_pages):
+                markdown_pages_to_add.append(
+                    {'name': page['name'], 'href': page['href']}
+                )
+
+        if not markdown_pages_to_add:
+            continue
+
+        markdown_pages_to_add = sorted(
+            markdown_pages_to_add,
+            key=lambda entry: entry['href'])
+
+        entry['items'] = markdown_pages_to_add + entry['items']
+        added_pages.update({
+            page['href'] for page in markdown_pages_to_add
+        })
+
+    if (top_level_pages := markdown_toc_yaml.get('/')) is None or (
+        top_level_pages and top_level_pages[0]['href'] != 'index.md'):
+        return added_pages, [pkg_toc_yaml]
+
+    added_pages.update({
+        page['href'] for page in top_level_pages
+    })
+
+    return added_pages, top_level_pages + pkg_toc_yaml
 
 
 def build_finished(app, exception):
@@ -1577,7 +1650,7 @@ def build_finished(app, exception):
     ensuredir(normalized_outdir)
 
     # Add markdown pages to the configured output directory.
-    find_markdown_pages(app, normalized_outdir)
+    markdown_utils.move_markdown_pages(app, normalized_outdir)
 
     pkg_toc_yaml = []
     # Used to record filenames dumped to avoid confliction
@@ -1700,23 +1773,6 @@ def build_finished(app, exception):
                 if obj.get('type', '') == 'package' and obj.get('kind', '') != 'distribution':
                     obj['kind'] = 'import'
 
-                try:
-                    if remove_inheritance_for_notfound_class:
-                        if 'inheritance' in obj:
-                            python_sdk_name = obj['uid'].split('.')[0]
-                            obj['inheritance'] = [n for n in obj['inheritance'] if not n['type'].startswith(python_sdk_name) or
-                                                  n['type'] in app.env.docfx_info_uid_types]
-                            if not obj['inheritance']:
-                                obj.pop('inheritance')
-
-                except NameError:
-                    pass
-
-                if 'source' in obj and (not obj['source']['remote']['repo'] or \
-                    obj['source']['remote']['repo'] == 'https://apidrop.visualstudio.com/Content%20CI/_git/ReferenceAutomation'):
-                        del(obj['source'])
-
-
                 # Extract any missing cross references where applicable.
                 # Potential targets are instances of full uid shown, or
                 # if we find a short form of the uid of one of current
@@ -1728,11 +1784,11 @@ def build_finished(app, exception):
                 #   (not from the same library)
                 #   google.cloud.aiplatform.AutoMLForecastingTrainingJob
 
-                current_name = obj["fullName"]
-                entry_names = sorted(app.env.docfx_uid_names.keys(), reverse=True)
+                current_object_name = obj["fullName"]
+                known_uids = sorted(app.env.docfx_uid_names.keys(), reverse=True)
                 # Currently we only need to look in summary, syntax and
                 # attributes for cross references.
-                search_cross_references(obj, current_name, entry_names)
+                search_cross_references(obj, current_object_name, known_uids)
 
             yaml_map[uid] = [yaml_data, references]
 
@@ -1786,13 +1842,26 @@ def build_finished(app, exception):
 
     sanitize_uidname_field(pkg_toc_yaml)
 
+    known_uids = {
+        uid.split('.')[-1]
+        for uid in app.env.docfx_uid_names
+    }
+
+    added_pages, pkg_toc_yaml = merge_markdown_and_package_toc(
+        pkg_toc_yaml, app.env.markdown_pages, known_uids)
+
+    # Remove unused pages after merging the table of contents.
+    if added_pages:
+        markdown_utils.remove_unused_pages(
+            added_pages, app.env.moved_markdown_pages, normalized_outdir)
+
     toc_file = os.path.join(normalized_outdir, 'toc.yml')
     with open(toc_file, 'w') as writable:
         writable.write(
             dump(
                 [{
                     'name': app.config.project,
-                    'items': [{'name': 'Overview', 'uid': 'project-' + app.config.project}] + app.env.markdown_pages + pkg_toc_yaml
+                    'items': pkg_toc_yaml
                 }],
                 default_flow_style=False,
             )
@@ -1852,25 +1921,6 @@ def build_finished(app, exception):
                 'fullname': item.get('uidname', ''),
                 'isExternal': False
             })
-    with open(index_file, 'w') as index_file_obj:
-        index_file_obj.write('### YamlMime:UniversalReference\n')
-        dump(
-            {
-                'items': [{
-                    'uid': 'project-' + app.config.project,
-                    'name': app.config.project,
-                    'fullName': app.config.project,
-                    'langs': ['python'],
-                    'type': 'package',
-                    'kind': 'distribution',
-                    'summary': 'Reference API documentation for `{}`.'.format(app.config.project),
-                    'children': index_children
-                }],
-                'references': index_references
-            },
-            index_file_obj,
-            default_flow_style=False
-        )
 
     '''
     # TODO: handle xref for other products.
